@@ -22,16 +22,29 @@
 
 #include <config.h>
 
-#include "cjs/jsapi-class.h"
-#include "cjs/jsapi-util-args.h"
-#include "cjs/jsapi-wrapper.h"
+#include <cairo-features.h>  // for CAIRO_HAS_PS_SURFACE
 #include <cairo.h>
-#include "cairo-private.h"
+
+#include <js/TypeDecls.h>
+
+#include "cjs/jsapi-util.h"
 
 #if CAIRO_HAS_PS_SURFACE
-#include <cairo-ps.h>
+#    include <cairo-ps.h>
+#    include <glib.h>
 
-static JSObject *gjs_cairo_ps_surface_get_proto(JSContext *);
+#    include <js/Class.h>
+#    include <js/PropertyDescriptor.h>  // for JSPROP_READONLY
+#    include <js/PropertySpec.h>
+#    include <js/RootingAPI.h>
+#    include <jsapi.h>  // for JS_NewObjectWithGivenProto
+
+#    include "cjs/jsapi-class.h"
+#    include "cjs/jsapi-util-args.h"
+#    include "cjs/macros.h"
+#    include "modules/cairo-private.h"
+
+[[nodiscard]] static JSObject* gjs_cairo_ps_surface_get_proto(JSContext*);
 
 GJS_DEFINE_PROTO_WITH_PARENT("PSSurface", cairo_ps_surface, cairo_surface,
                              JSCLASS_BACKGROUND_FINALIZE)
@@ -39,7 +52,7 @@ GJS_DEFINE_PROTO_WITH_PARENT("PSSurface", cairo_ps_surface, cairo_surface,
 GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_ps_surface)
 {
     GJS_NATIVE_CONSTRUCTOR_VARIABLES(cairo_ps_surface)
-    char *filename;
+    GjsAutoChar filename;
     double width, height;
     cairo_surface_t *surface;
 
@@ -54,14 +67,11 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(cairo_ps_surface)
     surface = cairo_ps_surface_create(filename, width, height);
 
     if (!gjs_cairo_check_status(context, cairo_surface_status(surface),
-                                "surface")) {
-        g_free(filename);
+                                "surface"))
         return false;
-    }
 
-    gjs_cairo_surface_construct(context, object, surface);
+    gjs_cairo_surface_construct(object, surface);
     cairo_surface_destroy(surface);
-    g_free(filename);
 
     GJS_NATIVE_CONSTRUCTOR_FINISH(cairo_ps_surface);
 
@@ -75,9 +85,11 @@ gjs_cairo_ps_surface_finalize(JSFreeOp *fop,
     gjs_cairo_surface_finalize_surface(fop, obj);
 }
 
+// clang-format off
 JSPropertySpec gjs_cairo_ps_surface_proto_props[] = {
-    JS_PS_END
-};
+    JS_STRING_SYM_PS(toStringTag, "PSSurface", JSPROP_READONLY),
+    JS_PS_END};
+// clang-format on
 
 JSFunctionSpec gjs_cairo_ps_surface_proto_funcs[] = {
     // restrictToLevel
@@ -98,19 +110,20 @@ JSObject *
 gjs_cairo_ps_surface_from_surface(JSContext       *context,
                                   cairo_surface_t *surface)
 {
-    g_return_val_if_fail(context != NULL, NULL);
-    g_return_val_if_fail(surface != NULL, NULL);
-    g_return_val_if_fail(cairo_surface_get_type(surface) == CAIRO_SURFACE_TYPE_PS, NULL);
+    g_return_val_if_fail(context, nullptr);
+    g_return_val_if_fail(surface, nullptr);
+    g_return_val_if_fail(
+        cairo_surface_get_type(surface) == CAIRO_SURFACE_TYPE_PS, nullptr);
 
     JS::RootedObject proto(context, gjs_cairo_ps_surface_get_proto(context));
     JS::RootedObject object(context,
         JS_NewObjectWithGivenProto(context, &gjs_cairo_ps_surface_class, proto));
     if (!object) {
         gjs_throw(context, "failed to create ps surface");
-        return NULL;
+        return nullptr;
     }
 
-    gjs_cairo_surface_construct(context, object, surface);
+    gjs_cairo_surface_construct(object, surface);
 
     return object;
 }
@@ -122,6 +135,6 @@ gjs_cairo_ps_surface_from_surface(JSContext       *context,
     gjs_throw(context,
         "could not create PS surface, recompile cairo and gjs with "
         "PS support.");
-    return NULL;
+    return nullptr;
 }
 #endif /* CAIRO_HAS_PS_SURFACE */
